package es.unican.docencia.y2011.eda.stacks.polishNotation;

import java.util.Stack;

/**
 * Clase que es capaz de calcular expresiones en notacion polaca
 * inversa
 * @author Pablo Snchez (p.sanchez@unican.es)
 * 	       http://personales.unican.es/sanchezbp	
 */
public class Calculadora {

	/**
	 * Funcion que acepta como entrada un array de cadenas de caracteres que
	 * representa una expresin aritmetica en notacin polaca inversa sobre 
	 * numeros naturales y devuelve su valor entero  
	 * @param expresion Vector de cadenas de caracteres que representa una 
	 * 					una expresin aritmetica en notacin polaca inversa sobre 
	 * 					numeros naturales
	 * 	Pre: La expresion debe ser correcta
	 * 		 Cada cadena dentro del vector debe ser:
	 * 			- o una cadena representando un numero natural
	 * 			- o un operador aritmtico ('+','-','*')  
	 * @return El valor de la expresion
	 */
	public static int calculaValor(String [] expresion) {
		
		// Creamos una nueva pila
		Stack<Integer> pila = new Stack<Integer>();
		
		// Recorremos la expresion hasta su final
		for(int i=0;i<expresion.length;i++) {
			// Si es operador, desapilamos los operandos, calculamos
			// y apilamos el resultado
			if (esOperador(expresion[i])) {
				// Asumimos que todos los operadores son binarios
				Integer op1 = pila.peek();
				pila.pop();
				Integer op2 = pila.peek();
				pila.pop();
				Integer result = opera(op1,op2,expresion[i]);
				pila.push(result);
			} else {
				// sino es operador, simplemente apilamos
				Integer op = Integer.parseInt(expresion[i]);
				pila.push(op);
			} // 
		} // for
			
		// retornamos el valor que queda almacenado en la pila
		return pila.peek();
	} // calculaValor
	
	//-----------------------------------------------------------------------//
	// 						Funciones auxiliares							 //
	//-----------------------------------------------------------------------//

	/**
	 * Indica si la cadena que se le pasa es un operador aritmetico valido
	 * @param s La cadena conteniendo el suspuesto operador
	 * @return Verdadero si la cadena contiene un operador valido. Falso en
	 * 		   caso contrario
	 */
	protected static boolean esOperador(String s) {
		return (s.equals("*") || s.equals("+") || s.equals("-"));
	} // esOperador
	
	/**
	 * Opera, segun el operador que se pasa como tercer parametro, los 
	 * operandos que se pasan como los dos primeros parametros 
	 * @param op1 Primer operando
	 * @param op2 Segundo operando
	 * @param op Operador
	 * @return El resultado de evaluar la expresion
	 */
	protected static Integer opera(Integer op1, Integer op2, String op) {
		Integer result = null;
		if (op.equals("+"))  {
			result = new Integer(op1 + op2);
		} else if (op.equals("-")) {
			result = new Integer(op1 - op2);
		} else if (op.equals("*")) {
			result = new Integer(op1 * op2);
		} // if
		return result;
	} // esOperador
	
} // Calculadora
